#ifndef _WMULTIJET_H_INCLUDED_
#define _WMULTIJET_H_INCLUDED_
#include <math.h>
#include <complex>
#include <list>
#include <CLHEP/Vector/LorentzVector.h>
#include <CLHEP/Random/Randomize.h>
#include <CLHEP/Random/RanluxEngine.h>
#include "fastjet/PseudoJet.hh"
#include "fastjet/ClusterSequence.hh"
#include "mstwpdf.h"
#include "GeneratorFlags.h"

typedef std::complex<double> COM;
typedef double FREAL;

#define printoutput 0
using namespace std;
using namespace CLHEP;

struct particle {
  int PID;
  int COL1;
  int COL2;
  int MOT1;
  int MOT2;
  int ISTUP;
  CLHEP::HepLorentzVector p;
};

class CWMultijet
{
private:
  // Generator flags:
  SGeneratorFlags GeneratorFlags;
  
  double wt; // The weight for the matching correction
  double matchreweight;

  std::vector<CLHEP::HepLorentzVector> partonmomenta;
  CLHEP::HepLorentzVector pd1; // the momentum of the particle in the W-decay
  CLHEP::HepLorentzVector pd2;// the momentum of the anti-particle in the W-decay
  particle lepton;
  particle alepton;
  particle Wboson;
  HepLorentzVector pa,pb,ti;
  std::vector<particle> LHparticles;
  ofstream LHoutput;
  long xsecpos;
  /* double ptmin=30.; */
  static int const GLUEINT=20; // the max number of gluons -1
  static double const beta0=11.-2.*5/3.;
  //  static int const GLUEINT=2; // the max number of gluons -1
  //  static int const GLUEINT=12; // the max number of gluons -1
  //  static double const RAPINT=10.0; // the maxlength of the rapidity interval
  double lambda; // the phase space slicing regularisation scale
  double thetaenu;
  static double const vud=0.97418,vus=0.2255,vub=0.00393,vcd=0.230,vcs=1.04,vcb=0.0412,vtd=0.0081,vts=0.0387,vtb=0.74; //These are the absolute values of the CKM Matrix elements as set by the PDG in 2009

  int aptype,bptype,aoutptype,boutptype; // a is backward, b forward
  bool wc; // Which parton emitted the W? a (0) or b (1)
  double xa,xb; // xa-, xb+
  double Qa,Qb;
  double matchingweight,matchingweight2;
  double ymax; // the maximum rapidity generated for any particle
  class c_mstwpdf * parton;
  void AddParton(double p[4]);
  void Def3jp1(),Def3jp2(),Def3jp3(),Def3jp4(),Def3jp5(),Def3jp6(),Def3jp7(),Def3jp8(),Def3jp9(),Def3jp10(),Def3jp11(),Def3jp12();
  void Def3jq1(),Def3jq2(),Def3jq3(),Def3jq4(),Def3jq5(),Def3jq6(),Def3jq7(),Def3jq8(),Def3jq9(),Def3jq10(),Def3jq11(),Def3jq12();
  /* void Def4jp1(),Def4jp2(),Def4jp3(),Def4jp4(),Def4jp5(),Def4jp6(),Def4jp7(),Def4jp8(),Def4jp9(),Def4jp10(),Def4jp11(),Def4jp12(); */
  double omega0(double q2, double lambda2);
  double beta0nc;
  COM PW(double s);
  FREAL pmg3[7][4];
  double MEt2(std::vector<CLHEP::HepLorentzVector> partons);
  double MEt2(std::vector<CLHEP::HepLorentzVector> partons, CLHEP::HepLorentzVector pa, CLHEP::HepLorentzVector pb,bool checkmomenta,fastjet::ClusterSequence * ev);
  double MG(std::vector<CLHEP::HepLorentzVector> partons, CLHEP::HepLorentzVector pa, CLHEP::HepLorentzVector pb, CLHEP::HepLorentzVector alp, CLHEP::HepLorentzVector lp);
  double MG3(std::vector<CLHEP::HepLorentzVector> partons, CLHEP::HepLorentzVector pa, CLHEP::HepLorentzVector pb, CLHEP::HepLorentzVector alp, CLHEP::HepLorentzVector lp);
  double MG4(std::vector<CLHEP::HepLorentzVector> partons, CLHEP::HepLorentzVector pa, CLHEP::HepLorentzVector pb, CLHEP::HepLorentzVector alp, CLHEP::HepLorentzVector lp);
  double VirtualCorrections();
  double GeneratePHSPpoint();
  double Generate2jphsppoint();
  double Generate3jphsppoint();
  double Generate4jphsppoint();
  /* double Generate2jNonFKLflavourconf_and_eval(); */
  /* double Generate3jNonFKLflavourconf_and_eval(); */
  double Generate2jNonFKLflavourconf_and_eval(double p[6][4]);
  double Generate3jNonFKLflavourconf_and_eval(double p[7][4]);
  double Generate4jNonFKLflavourconf_and_eval(double p[8][4]);
  double pdfpt(int beam, double a, double q,int t);
  double Halphas(double q);
  double GenerateFKLIncomingParticles(double xa,double ufa,double xb,double ufb);
  double GenerateIncomingParticles(double xa,double ufa,double xb,double ufb);
  void SetupLHInformation();
  int status; // the return status of an event
  int processtype; // the process type of an event - matching, 2,3,4 jets
  static const int nprocesstype=4; // resummed(&matched), 2jet, 3jet, 4jet
  // Scale setting parameters
  double Ht; // The scalar sum of transverse momenta in the event
  double renscale;  // Globally accessible scale variable.

  // jets, sorted by rapidity - used in choosing between CLs:
  std::vector<fastjet::PseudoJet> jets;

  std::vector<double> scales;
  std::vector<double> weights;

public:
  // constructor & destructor to be defined later
  virtual ~CWMultijet(void);
  CWMultijet(void);
  CWMultijet(string filename);
  CWMultijet(SGeneratorFlags GeneratorFlagsa);

  double getxa(),getxb(),getQa(),getQb(),getpdfta(),getpdftb();
  int getaptype(),getbptype();
  double pdfa,pdfb,pdfda,pdfdb,pdfuxa,pdfuxb,pdfdxa,pdfdxb,pdfga,pdfba,pdfgb,pdfsa,pdfsxa,pdfsb,pdfsxb,pdfca,pdfcb,pdfua,pdfub,pdfbb,pdfta,pdftb;
  SGeneratorFlags flags;
  double getcmsenergy();
  double weight();  // resummed (and matched ?)
  double weight1(); // 2-jet
  double weight2(); // 3-jet
  double weight3(); // 4-jet
  double Event();
  int getnp();
  HepLorentzVector getpartvn(int n);
  particle getlepton();
  particle getalepton();
  int getMstatus(void);
  HepRandom ran;
  int getstatus();
  int getprocesstype();
  double getthetaenu();
  void setstatus(int s);
  double GenerateEvent();
  void LHSaveEvent(void);
  double xsec[nprocesstype+1][2];
  double maxweight[nprocesstype+1];
  std::vector<particle> GetLHParticles(void);
  std::vector<double> GetWeights(void);
  double getmatchreweight();
};
#endif

